"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.StatusBarService = void 0;
const tslib_1 = require("tslib");
const mobx_1 = require("mobx");
const di_1 = require("@opensumi/di");
const events_1 = require("@opensumi/events");
const ide_core_browser_1 = require("@opensumi/ide-core-browser");
const layout_state_1 = require("@opensumi/ide-core-browser/lib/layout/layout-state");
const next_1 = require("@opensumi/ide-core-browser/lib/menu/next");
const raw_context_key_1 = require("@opensumi/ide-core-browser/lib/raw-context-key");
const services_1 = require("@opensumi/ide-core-browser/lib/services");
const ide_core_common_1 = require("@opensumi/ide-core-common");
let StatusBarService = class StatusBarService extends ide_core_browser_1.Disposable {
    constructor() {
        super();
        this.emitter = this.registerDispose(new events_1.EventEmitter());
        this.entries = new Map();
        this.disposableCollection = new Map();
        (0, mobx_1.makeObservable)(this);
    }
    /**
     * 获取背景颜色
     */
    getBackgroundColor() {
        if (this.appConfig.extensionDevelopmentHost) {
            return 'var(--kt-statusBar-extensionDebuggingBackground)';
        }
        return this.background;
    }
    getColor() {
        return this.foreground;
    }
    /**
     * 设置整个 Status Bar 背景颜色
     * @param color
     */
    setBackgroundColor(color) {
        this.background = color;
        this.emitter.emit('backgroundColor', color);
    }
    /**
     * 设置 Status Bar 所有文字颜色，当设置值为 undefined 时，文件将回复原有颜色配置
     * @param color
     */
    setColor(color) {
        this.foreground = color;
        this.emitter.emit('color', color);
    }
    // 暴露给其他地方获取配置数据以自定义渲染
    // 目前 scm 使用
    getElementConfig(entryId, entry) {
        // 如果有 command，覆盖自定义的 click 方法
        if (entry.command) {
            entry.onClick = this.onclick(entry);
        }
        entry.entryId = entryId;
        if ((0, ide_core_browser_1.isUndefined)(entry.id)) {
            entry.id = entryId;
        }
        return entry;
    }
    /**
     * 通过 id 获取关联的实例
     * @param id 状态栏 id
     * @returns
     */
    getEntriesById(id) {
        return this.entriesArray.filter((entry) => entry.id === id);
    }
    /**
     * 从 storage 中读取 state
     */
    getStorageState(id) {
        return this.layoutState.getState(layout_state_1.LAYOUT_STATE.STATUSBAR, {})[id];
    }
    setStorageState(id, state) {
        this.layoutState.setState(layout_state_1.LAYOUT_STATE.STATUSBAR, Object.assign(Object.assign({}, this.layoutState.getState(layout_state_1.LAYOUT_STATE.STATUSBAR, {})), { [id]: state }));
    }
    /**
     * 设置一个 StatusBar Item
     * @param id
     * @param entry
     */
    addElement(entryId, entry) {
        var _a, _b, _c, _d;
        const disposables = new ide_core_common_1.DisposableCollection();
        entry = this.getElementConfig(entryId, entry);
        const id = entry.id;
        // 优先读取 storage 数据
        const hidden = (_b = (_a = this.getStorageState(id)) === null || _a === void 0 ? void 0 : _a.hidden) !== null && _b !== void 0 ? _b : entry.hidden;
        entry.hidden = hidden;
        // 确保相同 id 只注册一次菜单me
        // 比如源码管理会注册多个状态栏元素，但菜单只会注册一个
        if (entry.name && this.getEntriesById(id).length === 0) {
            const toggleContextKey = new raw_context_key_1.RawContextKey(`${id}:toggle`, !hidden);
            toggleContextKey.bind(this.contextKeyService);
            // 如果当前状态栏在左侧，权重越大，排序越靠前，右侧反之
            // 负数为左，正数为右
            const order = entry.alignment === services_1.StatusBarAlignment.LEFT
                ? -1 * ((_c = entry.priority) !== null && _c !== void 0 ? _c : 0)
                : Number.MAX_SAFE_INTEGER - ((_d = entry.priority) !== null && _d !== void 0 ? _d : 0);
            const menuDisposer = this.menuRegistry.registerMenuItem(next_1.MenuId.StatusBarContext, {
                command: {
                    id: services_1.StatusBarCommand.toggleElement.id,
                    label: entry.name,
                },
                order,
                extraTailArgs: [entryId],
                toggledWhen: toggleContextKey.raw,
            });
            disposables.push(menuDisposer);
        }
        this.entries.set(entryId, entry);
        disposables.push(ide_core_browser_1.Disposable.create(() => {
            this.entries.delete(entryId);
        }));
        this.disposableCollection.set(entryId, disposables);
        return {
            dispose: () => {
                this.removeElement(entryId);
            },
            update: (entry) => {
                this.setElement(entryId, entry);
            },
        };
    }
    toggleElement(entryId) {
        const entry = this.entries.get(entryId);
        if (entry === null || entry === void 0 ? void 0 : entry.id) {
            const toggleContextKey = `${entry.id}:toggle`;
            const hidden = !entry.hidden;
            // 设置 toggle contextkey
            this.contextKeyService.createKey(toggleContextKey, !hidden);
            this.setStorageState(entry.id, {
                hidden,
            });
            // 确保修改了所有 id 相同的状态栏
            for (const toggleEntry of this.getEntriesById(entry.id)) {
                toggleEntry.hidden = !toggleEntry.hidden;
            }
        }
    }
    /**
     * 给指定 id 的元素设置属性
     * @param id
     * @param fields
     */
    setElement(entryId, fields) {
        const current = this.entries.get(entryId);
        if (current) {
            const entry = Object.assign(Object.assign({}, current), fields);
            this.entries.set(entryId, this.getElementConfig(entryId, entry));
        }
        else {
            throw new Error(`not found id is ${entryId} element`);
        }
    }
    /**
     * 删除一个元素
     * @param id
     */
    removeElement(entryId) {
        var _a;
        (_a = this.disposableCollection.get(entryId)) === null || _a === void 0 ? void 0 : _a.dispose();
        this.disposableCollection.delete(entryId);
    }
    /**
     * 数组形式的 entries
     * @readonly
     * @private
     * @type {StatusBarEntry[]}
     * @memberof StatusBarService
     */
    get entriesArray() {
        return Array.from(this.entries.values()).sort((left, right) => {
            const lp = left.priority || 0;
            const rp = right.priority || 0;
            return rp - lp;
        });
    }
    /**
     * Status Bar 左边的 Item
     * @readonly
     * @type {StatusBarEntry[]}
     * @memberof StatusBarService
     */
    get leftEntries() {
        return this.entriesArray.filter((entry) => !entry.hidden && entry.alignment === services_1.StatusBarAlignment.LEFT);
    }
    /**
     * Status Bar 右边的 Item
     *
     * @readonly
     * @type {StatusBarEntry[]}
     * @memberof StatusBarService
     */
    get rightEntries() {
        return this.entriesArray.filter((entry) => !entry.hidden && entry.alignment === services_1.StatusBarAlignment.RIGHT);
    }
    /**
     * command 转换 onClick 的方法
     * @param entry
     */
    onclick(entry) {
        return () => {
            if (entry.command) {
                const args = entry.arguments || [];
                this.commandService.executeCommand(entry.command, ...args);
            }
        };
    }
    get contextKeyService() {
        return this.registerDispose(this.globalContextKeyService.createScoped());
    }
    get contextMenu() {
        return this.registerDispose(this.menuService.createMenu(next_1.MenuId.StatusBarContext, this.contextKeyService));
    }
};
tslib_1.__decorate([
    mobx_1.observable,
    tslib_1.__metadata("design:type", Map)
], StatusBarService.prototype, "entries", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_common_1.CommandService),
    tslib_1.__metadata("design:type", Object)
], StatusBarService.prototype, "commandService", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_browser_1.AppConfig),
    tslib_1.__metadata("design:type", Object)
], StatusBarService.prototype, "appConfig", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(next_1.IMenuRegistry),
    tslib_1.__metadata("design:type", next_1.IMenuRegistry)
], StatusBarService.prototype, "menuRegistry", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(next_1.AbstractMenuService),
    tslib_1.__metadata("design:type", next_1.AbstractMenuService)
], StatusBarService.prototype, "menuService", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_browser_1.IContextKeyService),
    tslib_1.__metadata("design:type", Object)
], StatusBarService.prototype, "globalContextKeyService", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(layout_state_1.LayoutState),
    tslib_1.__metadata("design:type", layout_state_1.LayoutState)
], StatusBarService.prototype, "layoutState", void 0);
tslib_1.__decorate([
    mobx_1.action,
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [Object]),
    tslib_1.__metadata("design:returntype", void 0)
], StatusBarService.prototype, "setBackgroundColor", null);
tslib_1.__decorate([
    mobx_1.action,
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [Object]),
    tslib_1.__metadata("design:returntype", void 0)
], StatusBarService.prototype, "setColor", null);
tslib_1.__decorate([
    mobx_1.action,
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [String, Object]),
    tslib_1.__metadata("design:returntype", Object)
], StatusBarService.prototype, "getElementConfig", null);
tslib_1.__decorate([
    mobx_1.action,
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [String, Object]),
    tslib_1.__metadata("design:returntype", Object)
], StatusBarService.prototype, "addElement", null);
tslib_1.__decorate([
    mobx_1.action,
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [String]),
    tslib_1.__metadata("design:returntype", void 0)
], StatusBarService.prototype, "toggleElement", null);
tslib_1.__decorate([
    mobx_1.action,
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [String, Object]),
    tslib_1.__metadata("design:returntype", void 0)
], StatusBarService.prototype, "setElement", null);
tslib_1.__decorate([
    mobx_1.action,
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [String]),
    tslib_1.__metadata("design:returntype", void 0)
], StatusBarService.prototype, "removeElement", null);
tslib_1.__decorate([
    mobx_1.computed,
    tslib_1.__metadata("design:type", Array),
    tslib_1.__metadata("design:paramtypes", [])
], StatusBarService.prototype, "entriesArray", null);
tslib_1.__decorate([
    mobx_1.computed,
    tslib_1.__metadata("design:type", Array),
    tslib_1.__metadata("design:paramtypes", [])
], StatusBarService.prototype, "leftEntries", null);
tslib_1.__decorate([
    mobx_1.computed,
    tslib_1.__metadata("design:type", Array),
    tslib_1.__metadata("design:paramtypes", [])
], StatusBarService.prototype, "rightEntries", null);
tslib_1.__decorate([
    ide_core_common_1.memoize,
    tslib_1.__metadata("design:type", Object),
    tslib_1.__metadata("design:paramtypes", [])
], StatusBarService.prototype, "contextKeyService", null);
tslib_1.__decorate([
    ide_core_common_1.memoize,
    tslib_1.__metadata("design:type", Object),
    tslib_1.__metadata("design:paramtypes", [])
], StatusBarService.prototype, "contextMenu", null);
StatusBarService = tslib_1.__decorate([
    (0, di_1.Injectable)(),
    tslib_1.__metadata("design:paramtypes", [])
], StatusBarService);
exports.StatusBarService = StatusBarService;
//# sourceMappingURL=status-bar.service.js.map